/* This program tests function ReplaceAll().

   Input(keyboard): The names of the input and output files, and
                    the names of the target and replacement strings
   Output(screen):  Pprompts for input, confirmation message
------------------------------------------------------------------*/

#include <iostream.h>                    // cin, cout, <<, >>
#include <fstream.h>                     // fstream, <<, >>
#include <stdlib.h>                      // exit()

#include "Strings.h"                     // class Strings

int ReplaceAll(const Strings&,           // ReplaceAll() declaration
               const Strings&,
               const Strings&,
               const Strings&);

int main(void)
{
   cout << "\nThis program tests function ReplaceAll.\n";

   Strings
      InFileName,
      OutFileName,
      OldString,
      NewString;

   cout << "\nEnter name of input file: ";   // get input file name
   InFileName.GetLine(cin);

   cout << "\nReplace what string? ";        // get target string
   OldString.GetLine(cin);

   cout << "\nReplace it with what? ";       // get replacement string
   NewString.GetLine(cin);

   cout << "\nEnter name of output file: ";  // get output file name
   OutFileName.GetLine(cin);

   int                                       // replace all occurrences
      NumReplaced = ReplaceAll(OldString,    //   of OldString
                               InFileName,   //   in InFileName
                               NewString,    // with NewString
                               OutFileName); // in OutFileName

   cout << "\nThere were a total of "        // report results
        << NumReplaced << " occurrences replaced.\n\n";

	return 0;
}

/* This function uses class Strings to replace all occurrences of a
   string in a given input file with another string.

   Receive:             Target, the string to be replaced
                        InFileName, the name of the input file
                        Replacement, the replacement string
                        OutFileName, the name of the output file
   Input(InFileName):   The characters in the file, line by line
   Output(OutFileName): The input characters, with occurrences of
                        Target replaced by Replacement
   Return:              The number of replacements performed
-------------------------------------------------------------------*/

int ReplaceAll(const Strings& Target,
               const Strings& InFileName,
               const Strings& Replacement,
               const Strings& OutFileName)
{
   if (Target == Replacement)            // no change required
      return 0;

   int
      Occurrences = 0;                  // replacement counter

   fstream                              // open files for I/O
      InStream(InFileName, ios::in),
      OutStream(OutFileName, ios::out);

   if (InStream.fail())                 //  check for success
   {
      cerr << "\n*** Unable to open file '"
           << InFileName << "'!\n";
      exit (1);
   }
   if (OutStream.fail())
   {
      cerr << "\n*** Unable to open file \'"
           << OutFileName << "\'!\n";
      exit (1);
   }

   Strings
      Line;                            // stores a line of input
   int
      Location,                        // position of Target in Line
      TargetLength = Target.Length();  // length of Target

   Line.GetLine(InStream);             // get the first Line

   while (!InStream.eof())             // while not end-of-file,
   {
      Location = Line.Position(Target);//   search Line for Target

      while (Location >= 0)            //   while Target in Line
      {
         Occurrences++;                //     increment counter
         Line.Replace(Location,        //     replace Target
                      Location + TargetLength - 1,
                      Replacement);
         Location = Line.Position(Target);//   search Line for Target
      }                                //   end while

      OutStream << Line << endl;       //   send the line to OutStream
                                       //   reset for next repetition
      Line.GetLine(InStream);          //   get the next Line
   }                                   // end while

   InStream.close();                   // close the two streams
   OutStream.close();

   return Occurrences;                 // return # of replacements
}
